//! A connection for a recieving Misfin server
pub mod builder;
pub mod error;
pub mod verifier;

use {
    crate::{
        prelude::{ClientCertificateStore, Id, MailStore, MessageParser},
        request::Request,
        response::Response,
    },
    std::{
        io::Read,
        sync::{Arc, Mutex},
    },
};

pub use self::{
    builder::Builder,
    error::Error,
    verifier::{FingerPrintStore, Verifier},
};

#[derive(Debug)]
pub struct Connection {
    pub inner: rustls::ServerConnection,
}

impl Connection {
    pub fn handle_request<C: ClientCertificateStore + FingerPrintStore, S: MailStore>(
        &mut self,
        verifier: Verifier<C>,
        mailstore: Arc<Mutex<S>>,
    ) -> Result<Response, Error> {
        let mut buf: Vec<u8> = Vec::with_capacity(2048);
        self.inner.reader().read_to_end(&mut buf)?;
        let raw = String::from_utf8(buf)?;
        let request: Request = raw.parse()?;
        let res = if let Ok(mut store) = mailstore.lock() {
            if store.has_mailuser(&request.recipient.to_string()) {
                let id = Id::new()?;
                let msg = MessageParser::new(&id.to_string()).parse(&request.message)?;
                store.add_message(&request.recipient.to_string(), "Inbox", msg)
                    .map_err(|e| Error::Storage(e.to_string()))?;
                true
            } else {
                false
            }
        } else {
            false
        };
        todo!()
    }
}
