use {
    crate::prelude::ParseMailboxError,
    std::{ffi::NulError, fmt, io, str::Utf8Error},
};

#[derive(Debug)]
pub enum Error {
    Io(io::Error),
    MailBox(ParseMailboxError),
    FFi(NulError),
    Permissions(pw::Error),
    Utf8,
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Io(e) => write!(f, "Filesystem error: {e}"),
            Self::FFi(e) => write!(f, "Filesystem error: {e}"),
            Self::MailBox(e) => write!(f, "Filesystem error: {e}"),
            Self::Permissions(e) => write!(f, "Filesystem error: {e}"),
            Self::Utf8 => write!(f, "Filesystem error: Utf8 failure"),
        }
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Io(e) => Some(e),
            Self::FFi(e) => Some(e),
            Self::MailBox(e) => Some(e),
            Self::Permissions(e) => Some(e),
            _ => None,
        }
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<NulError> for Error {
    fn from(value: NulError) -> Self {
        Self::FFi(value)
    }
}

impl From<ParseMailboxError> for Error {
    fn from(value: ParseMailboxError) -> Self {
        Self::MailBox(value)
    }
}

impl From<pw::Error> for Error {
    fn from(value: pw::Error) -> Self {
        Self::Permissions(value)
    }
}

impl From<Utf8Error> for Error {
    fn from(_value: Utf8Error) -> Self {
        Self::Utf8
    }
}
